﻿'DI_155.ChannelArray map:
'ChannelArray(0-3) analog inputs 0-3
'ChannelArray(4-7) not used
'ChannelArray(8) all digital inputs (all four bits)
'ChannelArray(9) frequency input
'ChannelArray(10) count input
'ChannelArray(11) digital outputs (all four bits)

Imports dataq.simple

Public Class frmDI_155Example
    Dim DataqDeviceArray() As DataqDevice 'array containing all connected devices
    Dim WithEvents DI_155 As DataqDevice 'this is the primary device we'll work with
    Dim CollectedData(11) As Double 'dimension the array that will hold acquired data
    Dim ChannelConfig(11) As String 'array to retain the channel configuration defined via the GUI

    Private Sub frmDI_155Example_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'upon loading the application do some device discovery, connection, and housekeeping

        'Discover connected devices
        DataqDeviceArray = Discovery.DiscoverAllDevices

        'Use only the first device in the discovery array
        'Bail if the first arrary position is empty
        If DataqDeviceArray.Length = 0 Then
            MessageBox.Show("No compatible DATAQ Instruments devices found.", _
                            "Missing DATAQ Instruments hardware", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Error)
            End

            'get here if the first array position contains something
        Else
            DI_155 = DataqDeviceArray(0) 'assign DataqDevice DI_155 to the first device in the array and...
            DI_155.Connect() '...connect to it
        End If

        'display model-specific information
        lblModel.Text = DI_155.Model
        lblSerial.Text = DI_155.SerialNumber
        lblPort.Text = DI_155.ComPort

        'Bail if the connected device is not model DI-155
        If DI_155.Model <> "DI-155" Then
            MessageBox.Show("This program works only with DATAQ Instruments model DI-155", _
                           "Incompatible Device (" + DI_155.Model + ")", _
                           MessageBoxButtons.OK, _
                           MessageBoxIcon.Error)
            End
        End If

        'load up ChannelConfig list with a default analog channel configuration
        For i As Integer = 0 To 3
            ChannelConfig(i) = "10" 'set all analog channels to the ±10 V full scale range
        Next
        cbAnalogConfig.SelectedIndex = 5 'display ±10 V fs to be consistent

        'ChannelConfig(4-7) are unused
        'now define the default settings for the digital inputs
        ChannelConfig(8) = "Off" 'turn off all discrete bits
        ChannelConfig(9) = "Off" ' turn off rate
        ChannelConfig(10) = "Off" ' turn off count


        'display "Off" to be consistent for all digital inputs
        cbDin0_3.SelectedIndex = 0
        cbDin2.SelectedIndex = 0
        cbDin3.SelectedIndex = 0

        'default is to configure an analog channel so hide the digital config combo boxes
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = False
        'everthing's good, so create an event handler that fires when new DI-155 data is available
        AddHandler DI_155.NewData, AddressOf GetDI155Data
    End Sub
    Private Sub GetDI155Data()
        'this is the event handler for DI-155 data. Get here when new data is available during a scan
        Dim Scans As Integer = DI_155.NumberOfScansAvailable
        Dim Channels As Short = DI_155.NumberOfChannelsEnabled
        Dim DI_155_Data(Scans * Channels - 1) As Double 'will hold all data for the scan
        Dim ResponseString As String = ""

        'Move data into temporary array
        DI_155.GetInterleavedScaledData(DI_155_Data, 0, Scans)
        'now move it to the list box
        For Row As Integer = 0 To Scans - 1
            For Column As Short = 0 To Channels - 1
                ResponseString = ResponseString + DI_155_Data(Column + (Row * Channels)).ToString("'+'00.000;'-'00.000")
                If Column <> Channels - 1 Then
                    ResponseString = ResponseString + ", "
                End If
            Next
            UpdateList(ResponseString + vbCrLf) 'output to the list box
            ResponseString = "" 'reset ResponseString
        Next
    End Sub
    Private Sub btnStart_Click(sender As System.Object, e As System.EventArgs) Handles btnStart.Click
        'this sub configures the DI-155 scan list. 
        'DI_155.ChannelArray map:
        'ChannelArray(0-3) analog inputs 0-3 (or 1-4 for normal people as per the device sticker)
        'ChannelArray(4-7) not used
        'ChannelArray(8) all digital inputs (all four bits)
        'ChannelArray(9) frequency input
        'ChannelArray(10) count input

        Dim Range As New dataq.Range(Of Double) 'holds the FSR of an analog channel or the rate channel

        'first, some GUI housekeeping
        btnStart.Enabled = False
        btnStop.Enabled = True
        cbSampleRate.Enabled = False
        lbOutput.Items.Clear()
        lblSamplingRate.Text = ""
        lblRealTime.Text = ""
        cbAnalogConfig.Enabled = False
        cbDin0_3.Enabled = False
        cbDin2.Enabled = False
        cbDin3.Enabled = False
        rbAin1.Enabled = False
        rbAin2.Enabled = False
        rbAin3.Enabled = False
        rbAin4.Enabled = False
        rbDin0_1.Enabled = False
        rbRate.Enabled = False
        rbCount.Enabled = False

        'Configure the DI-155's scan list for analog channels
        'do the following for each of the four analog channels:
        For Channel As Integer = 0 To 3
            'first disable the indicated channel
            DI_155.ChannelArray(Channel).Enabled = False

            'now configure the channel as defined by array ChannelConfig
            If ChannelConfig(Channel) <> "Off" Then
                'not OFF so must be on at the indicated range
                Range.Maximum = ChannelConfig(Channel) 'define the Range variable...
                DI_155.ChannelArray(Channel).InputRange = Range '...and set it
                DI_155.ChannelArray(Channel).Enabled = True ' finally, enable the channel for acq
            End If
        Next

        'now set up the digital input channels

        For Channel As Integer = 8 To 10
            DI_155.ChannelArray(Channel).Enabled = False 'disable the channel first to aid management
            Select Case True
                Case Channel = 8
                    'this is handles the acquisition of all digital input bit states (0-3)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on, so enable 
                        DI_155.ChannelArray(Channel).Enabled = True
                    End If
                Case Channel = 9
                    'this handles the rate input bit (Din2)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on
                        Range.Maximum = ChannelConfig(Channel) 'define the Range variable...
                        DI_155.ChannelArray(Channel).InputRange = Range '...and set it
                        DI_155.ChannelArray(Channel).Enabled = True ' finally, enable the channel for acq
                    End If
                Case Channel = 10
                    'this handles the count input bit (Din3)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on, so enable it
                        DI_155.ChannelArray(Channel).Enabled = True
                    End If

            End Select
        Next

        'set the sampling rate
        'Note: the class will always select the sampling rate closest to the one selected constrained
        'by the number of enabled channels. For a full treatment of how to configure DI-155 sampling rate
        'refer to the instrument's prototocol document:
        'http://www.dataq.com/resources/pdfs/misc/di-155-protocol.pdf
        DI_155.SampleRatePerChannel = Val(cbSampleRate.Text)

        'display the actual sampling rate per channel
        lblSamplingRate.Text = DI_155.SampleRatePerChannel.ToString("F3")

        DI_155.NewDataMinimum = 1 'set the number of scans to acquire before the NewData event fires
        DI_155.Start() 'start scanning
        SetDigitalOutputs() 'write approrpiate values to the digital output port
    End Sub
    Private Sub SetDigitalOutputs()
        'Construct a value between 0-15 (4-bit) to write to the digital output port as a function of
        'checked (enabled) bits. When a bit is checked the output is driven low (each is a low-true output)
        Dim DigitalOutValue As Short = 0
        For Port As Short = 0 To 3
            Select Case True
                Case (Port = 0) And ckbDout0.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 1) And ckbDout1.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 2) And ckbDout2.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 3) And ckbDout3.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
            End Select
        Next
        DI_155.ChannelArray(11).Write(DigitalOutValue) 'write the value to the ports asynchronously during data acq
        'use the asynchrounous feature for real time alarms and such. 
    End Sub
    Private Sub btnStop_Click(sender As System.Object, e As System.EventArgs) Handles btnStop.Click
        'stop button clicked so stop the DI-155 and perform some GUI housekeeping
        DI_155.Stop()
        'gui housekeeping stuff
        btnStart.Enabled = True
        btnStop.Enabled = False
        cbSampleRate.Enabled = True
        cbAnalogConfig.Enabled = False
        cbDin0_3.Enabled = False
        cbDin2.Enabled = False
        cbDin3.Enabled = False

        cbAnalogConfig.Enabled = True
        cbDin0_3.Enabled = True
        cbDin2.Enabled = True
        cbDin3.Enabled = True
        rbAin1.Enabled = True
        rbAin2.Enabled = True
        rbAin3.Enabled = True
        rbAin4.Enabled = True
        rbDin0_1.Enabled = True
        rbRate.Enabled = True
        rbCount.Enabled = True
    End Sub
    'UpdateList is called from a different thread than the main program, so we need to delegate
    Private Delegate Sub UpdateListDelegate(ByVal ResponseString As String) 'create new class UpdateListDelegate
    Private Sub UpdateList(ByVal ResponseString As String)
        If Me.InvokeRequired Then
            Me.Invoke(New UpdateListDelegate(AddressOf UpdateList), ResponseString) 'invoke delegate then...
        Else
            lbOutput.Items.Add(ResponseString) '...output string to list box
            lblRealTime.Text = ResponseString
        End If
    End Sub
    Private Sub rbResetCount_Click(sender As System.Object, e As System.EventArgs) Handles rbResetCount.Click
        DI_155.ChannelArray(10).Reset()
    End Sub
    'the below is all mundane GUI management stuff, none of which accesses the DI-155 directly. 
    Private Sub cbChannelConfig_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbAnalogConfig.SelectedIndexChanged
        'When combobox cbChannelConfig changes, update array ChannelConfig accordingly
        'as a function of the enabled channel
        Select Case True
            Case rbAin1.Checked
                ChannelConfig(0) = cbAnalogConfig.Text
            Case rbAin2.Checked
                ChannelConfig(1) = cbAnalogConfig.Text
            Case rbAin3.Checked
                ChannelConfig(2) = cbAnalogConfig.Text
            Case rbAin4.Checked
                ChannelConfig(3) = cbAnalogConfig.Text
        End Select
    End Sub

    Private Sub rbChan1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin1.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = False
        'diplay the configuration of analog channel 1 in combo box cbChannelConfig
        If rbAin1.Checked Then
            cbAnalogConfig.Text = ChannelConfig(0)
        End If
    End Sub

    Private Sub rbChan2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin2.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = False
        'diplay the configuration of analog channel 2 in combo box cbChannelConfig
        If rbAin2.Checked Then
            cbAnalogConfig.Text = ChannelConfig(1)
        End If
    End Sub

    Private Sub rbChan3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin3.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = False
        'diplay the configuration of analog channel 3 in combo box cbChannelConfig
        If rbAin3.Checked Then
            cbAnalogConfig.Text = ChannelConfig(2)
        End If
    End Sub

    Private Sub rbChan4_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin4.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        If rbAin4.Checked Then
            cbAnalogConfig.Text = ChannelConfig(3)
        End If
    End Sub

    Private Sub btnClear_Click(sender As System.Object, e As System.EventArgs) Handles btnClear.Click
        'Clear the output list box
        lbOutput.Items.Clear()
        lblRealTime.Text = ""
    End Sub

    Private Sub cbDin0_1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbDin0_3.SelectedIndexChanged
        'When combobox cbDin0_3 changes, update array ChannelConfig accordingly
        ChannelConfig(8) = cbDin0_3.Text
    End Sub

    Private Sub rbDin0_1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbDin0_1.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = True
        cbDin2.Visible = False
        cbDin3.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbDin0_3.Text = ChannelConfig(8)
    End Sub

    Private Sub rbDin2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbRate.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = False
        cbDin2.Visible = True
        cbDin3.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbDin2.Text = ChannelConfig(9)
    End Sub

    Private Sub rbDin3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbCount.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = False
        cbDin2.Visible = False
        cbDin3.Visible = True
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbDin3.Text = ChannelConfig(10)
    End Sub

    Private Sub cbDin2_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbDin2.SelectedIndexChanged
        ChannelConfig(9) = cbDin2.Text
    End Sub

    Private Sub cbDin3_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbDin3.SelectedIndexChanged
        ChannelConfig(10) = cbDin3.Text
    End Sub

    Private Sub ckbDout0_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout0.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout1.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout2.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout3.CheckedChanged
        SetDigitalOutputs()
    End Sub
End Class
